@@ -5,6 +5,7 @@ import os
 from pmb.core.context import get_context
 from pmb.core.pkgrepo import pkgrepo_default_path
 from pmb.helpers import logging
+from pmb.helpers.exceptions import NonBugError
 import pmb.aportgen.busybox_static
 import pmb.aportgen.core
 import pmb.aportgen.device
@@ -13,7 +14,6 @@ import pmb.aportgen.linux
 import pmb.aportgen.musl
 import pmb.aportgen.grub_efi
 import pmb.config
-from pmb.types import AportGenEntry
 import pmb.helpers.cli
 
 
@@ -39,7 +39,7 @@ def properties(
     pkgname: str,
     folder: Path | None = None,
     device_category: pmb.helpers.devices.DeviceCategory | None = None,
-) -> tuple[str, Path, AportGenEntry]:
+) -> tuple[str, Path, bool]:
     """
     Get the properties for the aport generator, based on the pkgname prefix.
 
@@ -49,12 +49,12 @@ def properties(
     :param folder: optional base folder override
     :param device_category: optional device category for device/linux aports
 
-    :returns: (prefix, folder, options)
+    :returns: (prefix, folder, confirm_overwrite)
     """
 
     for prefix in ["busybox-static", "gcc", "musl", "grub-efi"]:
         if pkgname.startswith(prefix):
-            return (prefix, folder or Path("cross"), {"confirm_overwrite": False})
+            return (prefix, folder or Path("cross"), False)
 
     for prefix in ["device", "linux"]:
         if pkgname.startswith(prefix):
@@ -62,18 +62,17 @@ def properties(
                 folder = (
                     Path("device") / str(device_category) if device_category else Path("device")
                 )
-            return (prefix, folder, {"confirm_overwrite": True})
+            return (prefix, folder, True)
 
     logging.info(
-        "NOTE: aportgen is for generating postmarketOS specific"
-        " aports, such as the cross-compiler related packages"
-        " or the linux kernel fork packages."
+        "NOTE: aportgen is for generating postmarketOS specific aports, such as the cross-compiler "
+        "related packages or the linux kernel fork packages."
     )
     logging.info(
         "NOTE: If you wanted to package new software in general, try"
         " 'pmbootstrap newapkbuild' to generate a template."
     )
-    raise ValueError("No generator available for " + pkgname + "!")
+    raise NonBugError(f"No generator available for {pkgname}!")
 
 
 def generate(
@@ -83,23 +82,21 @@ def generate(
     folder: Path | None = None,
     device_category: pmb.helpers.devices.DeviceCategory | None = None,
 ) -> None:
-    options: AportGenEntry
-
     if pkgname.startswith(("device", "linux")) and not device_category:
         device_category = pmb.config.ask_for_mainline_downstream()
 
     if fork_alpine:
-        prefix, folder, options = (pkgname, Path("temp"), {"confirm_overwrite": True})
+        prefix, folder, confirm_overwrite = (pkgname, Path("temp"), True)
     else:
-        prefix, folder, options = properties(pkgname, folder, device_category)
+        prefix, folder, confirm_overwrite = properties(pkgname, folder, device_category)
     config = get_context().config
     path_target = pkgrepo_default_path() / folder / pkgname
 
     # Confirm overwrite
-    if options["confirm_overwrite"] and os.path.exists(path_target):
+    if confirm_overwrite and os.path.exists(path_target):
         logging.warning(f"WARNING: Target folder already exists: {path_target}")
         if not pmb.helpers.cli.confirm("Continue and overwrite?"):
-            raise RuntimeError("Aborted.")
+            raise NonBugError("Aborted.")
 
     aportgen = config.work / "aportgen"
 
@@ -110,9 +107,7 @@ def generate(
             pkgname, retain_branch=fork_alpine_retain_branch
         )
         pmb.helpers.run.user(["cp", "-r", upstream, aportgen])
-        pmb.aportgen.core.rewrite(
-            pkgname, replace_simple={"# Contributor:*": None, "# Maintainer:*": None}
-        )
+        pmb.aportgen.core.rewrite(pkgname)
     else:
         match prefix:
             case "busybox-static":
@@ -128,7 +123,8 @@ def generate(
             case "grub-efi":
                 pmb.aportgen.grub_efi.generate(pkgname)
             case "linux":
-                pmb.aportgen.linux.generate(pkgname)
+                # Ignore mypy error; see note for "device" case above.
+                pmb.aportgen.linux.generate(pkgname, device_category)  # type: ignore[arg-type]
             case "musl":
                 pmb.aportgen.musl.generate(pkgname)
             case _:
