@@ -33,7 +33,8 @@ apk_keys_path: Path = pmb_src / "pmb/data/keys"
 # (which may contain a vulnerable apk/openssl, and allows an attacker to
 # exploit the system!)
 apk_tools_min_version = {
-    "edge": "3.0.0_rc5_git20250819",
+    "edge": "3.0.2-r0",
+    "v3.23": "3.0.2-r0",
     "v3.22": "2.14.9-r3",
     "v3.21": "2.14.6-r3",
     "v3.20": "2.14.4-r1",
@@ -163,6 +164,7 @@ host_path = os.environ["PATH"] + ":/usr/sbin/"
 # Use no more than one dir after /mnt/pmbootstrap, see remove_mnt_pmbootstrap.
 chroot_mount_bind = {
     "/proc": "/proc",
+    "/sys": "/sys",
     "$WORK/cache_apk_$ARCH": "/var/cache/apk",
     "$WORK/cache_appstream/$ARCH/$CHANNEL": "/mnt/appstream-data",
     "$WORK/cache_ccache_$ARCH": "/mnt/pmbootstrap/ccache",
@@ -253,11 +255,15 @@ apkbuild_package_attributes = {
     # postmarketos-base and UI meta-packages can define the default package
     # to select during "_pmb_select".
     "_pmb_default": {"array": True},
+    # kernel packages can specify one or more defconfig to use when generating
+    # the full config
+    "_defconfig": {"array": True},
 }
 
 # Variables in APKBUILD files that get parsed
 apkbuild_attributes = {
     **apkbuild_package_attributes,
+    "maintainer": {},
     "arch": {"array": True},
     "depends_dev": {"array": True},
     "makedepends": {"array": True},
