@@ -6,11 +6,10 @@ from collections.abc import Sequence
 from pathlib import Path
 from typing import Literal
 
-import pmb.chroot
 import pmb.config.pmaports
 from pmb.core.arch import Arch
 from pmb.core.chroot import Chroot
-from pmb.types import PathString
+from pmb.types import PathString, RunOutputTypePopen
 import pmb.helpers.cli
 import pmb.helpers.repo
 import pmb.helpers.run
@@ -102,7 +101,7 @@ def _prepare_fifo() -> Path:
     return fifo
 
 
-def _create_command_with_progress(command: list[str], fifo: Path) -> list[str]:
+def _create_command_with_progress(command: Sequence[str], fifo: Path) -> list[str]:
     """Build a full apk command from a subcommand, set up to redirect progress into a fifo.
 
     :param command: apk subcommand in list form
@@ -132,7 +131,7 @@ def _compute_progress(line: str) -> float:
     return cur / tot if tot > 0 else 0
 
 
-def _apk_with_progress(command: list[str]) -> None:
+def _apk_with_progress(command: Sequence[str]) -> None:
     """Run an apk subcommand while printing a progress bar to STDOUT.
 
     :param command: apk subcommand in list form
@@ -141,17 +140,19 @@ def _apk_with_progress(command: list[str]) -> None:
     fifo = _prepare_fifo()
     command_with_progress = _create_command_with_progress(command, fifo)
     log_msg = " ".join(command)
-    with pmb.helpers.run.root(["cat", fifo], output="pipe") as p_cat:
-        with pmb.helpers.run.root(command_with_progress, output="background") as p_apk:
-            while p_apk.poll() is None:
-                p_cat_stdout = p_cat.stdout
-                if p_cat_stdout is None:
-                    raise RuntimeError("cat process had no stdout?")
-                line = p_cat_stdout.readline().decode("utf-8")
-                progress = _compute_progress(line)
-                pmb.helpers.cli.progress_print(progress)
-            pmb.helpers.cli.progress_flush()
-            pmb.helpers.run_core.check_return_code(p_apk.returncode, log_msg)
+    with (
+        pmb.helpers.run.root(["cat", fifo], output=RunOutputTypePopen.PIPE) as p_cat,
+        pmb.helpers.run.root(command_with_progress, output=RunOutputTypePopen.BACKGROUND) as p_apk,
+    ):
+        while p_apk.poll() is None:
+            p_cat_stdout = p_cat.stdout
+            if p_cat_stdout is None:
+                raise RuntimeError("cat process had no stdout?")
+            line = p_cat_stdout.readline().decode("utf-8")
+            progress = _compute_progress(line)
+            pmb.helpers.cli.progress_print(progress)
+        pmb.helpers.cli.progress_flush()
+        pmb.helpers.run_core.check_return_code(p_apk.returncode, log_msg)
 
 
 def _prepare_cmd(command: Sequence[PathString], chroot: Chroot | None) -> list[str]:
@@ -164,10 +165,10 @@ def _prepare_cmd(command: Sequence[PathString], chroot: Chroot | None) -> list[s
     # Our _apk_with_progress() wrapper also need --no-progress, since all that does is
     # prevent apk itself from rendering progress bars. We instead want it to tell us
     # the progress so we can render it. So we always set --no-progress.
-    _command: list[str] = [str(config.work / "apk.static"), "--no-progress"]
+    command_: list[str] = [str(config.work / "apk.static"), "--no-progress"]
     if chroot:
         cache_dir = config.work / f"cache_apk_{chroot.arch}"
-        _command.extend(
+        command_.extend(
             [
                 "--root",
                 str(chroot.path),
@@ -177,26 +178,26 @@ def _prepare_cmd(command: Sequence[PathString], chroot: Chroot | None) -> list[s
         )
 
         if os.getenv("PMB_APK_NO_CACHE") == "1":
-            _command.extend(["--no-cache"])
+            command_.extend(["--no-cache"])
         else:
-            _command.extend(["--cache-dir", str(cache_dir)])
+            command_.extend(["--cache-dir", str(cache_dir)])
 
         local_repos = pmb.helpers.repo.urls(
             user_repository=config.work / "packages", mirrors_exclude=True
         )
         for repo in local_repos:
-            _command.extend(["--repository", repo])
+            command_.extend(["--repository", repo])
     if get_context().offline:
-        _command.append("--no-network")
+        command_.append("--no-network")
 
     for c in command:
-        _command.append(os.fspath(c))
+        command_.append(os.fspath(c))
 
         # Always be non-interactive
         if c == "add":
-            _command.append("--no-interactive")
+            command_.append("--no-interactive")
 
-    return _command
+    return command_
 
 
 def run(command: Sequence[PathString], chroot: Chroot, with_progress: bool = True) -> None:
@@ -207,28 +208,28 @@ def run(command: Sequence[PathString], chroot: Chroot, with_progress: bool = Tru
     :param chroot: chroot to run the command in
     :raises RuntimeError: when the apk command fails
     """
-    _command = _prepare_cmd(command, chroot)
+    command_ = _prepare_cmd(command, chroot)
 
     # Sanity checks. We should avoid accidentally writing to
     # /var/cache/apk on the host!
     if "add" in command:
-        if "--no-interactive" not in _command:
+        if "--no-interactive" not in command_:
             raise RuntimeError(
                 "Encountered an 'apk add' command without --no-interactive! This is a bug."
             )
-        if os.getenv("PMB_APK_NO_CACHE") == "1" and "--no-cache" not in _command:
+        if os.getenv("PMB_APK_NO_CACHE") == "1" and "--no-cache" not in command_:
             raise RuntimeError(
                 "Encountered an 'apk add' command without --no-cache! This is a bug."
             )
-        if os.getenv("PMB_APK_NO_CACHE") != "1" and "--cache-dir" not in _command:
+        if os.getenv("PMB_APK_NO_CACHE") != "1" and "--cache-dir" not in command_:
             raise RuntimeError(
                 "Encountered an 'apk add' command without --cache-dir! This is a bug."
             )
 
     if with_progress:
-        _apk_with_progress(_command)
+        _apk_with_progress(command_)
     else:
-        pmb.helpers.run.root(_command)
+        pmb.helpers.run.root(command_)
 
 
 def cache_clean(arch: Arch) -> None:
@@ -279,21 +280,19 @@ def cache_clean(arch: Arch) -> None:
     ]
 
     command += ["cache", "clean"]
-    _command = _prepare_cmd(command, None)
+    command_ = _prepare_cmd(command, None)
 
     pmb.helpers.apk_static.init()
-    pmb.helpers.run.root(_command)
+    pmb.helpers.run.root(command_)
 
 
-def check_outdated(version_installed: str, action_msg: str) -> None:
+def check_outdated(version_installed: str) -> None:
     """Check if the provided alpine version is outdated.
 
     This depends on the alpine mirrordir (edge, v3.12, ...) related to currently checked out
     pmaports branch.
 
     :param version_installed: currently installed apk version, e.g. "2.12.1-r0"
-    :param action_msg: string explaining what the user should do to resolve
-                       this
     :raises: RuntimeError if the version is outdated
     """
     channel_cfg = pmb.config.pmaports.read_config_channel()
@@ -304,7 +303,5 @@ def check_outdated(version_installed: str, action_msg: str) -> None:
         return
 
     raise RuntimeError(
-        "Found an outdated version of the 'apk' package"
-        f" manager ({version_installed}, expected at least:"
-        f" {version_min}). {action_msg}"
+        f"Found an outdated version of the 'apk' package manager ({version_installed}, expected at least: {version_min})"
     )
