@@ -9,7 +9,6 @@ from pmb.helpers import logging
 from pmb.helpers.exceptions import NonBugError
 import os
 from pathlib import Path
-import platform
 import re
 import signal
 import shlex
@@ -20,11 +19,9 @@ import pmb.build
 import pmb.chroot
 import pmb.chroot.apk
 import pmb.chroot.other
-import pmb.chroot.initfs
-import pmb.config
 import pmb.config.pmaports
 import pmb.install.losetup
-from pmb.types import Env, PathString, PmbArgs
+from pmb.types import Env, PathString, PmbArgs, RunOutputTypeDefault
 import pmb.helpers.run
 import pmb.parse.cpuinfo
 from pmb.core import Chroot, ChrootType
@@ -62,7 +59,7 @@ def which_qemu(arch: Arch) -> str:
     """
     Finds the qemu executable or raises an exception otherwise
     """
-    executable = "qemu-system-" + arch.qemu()
+    executable = "qemu-system-" + arch.qemu_system()
     if shutil.which(executable):
         return executable
     else:
@@ -124,6 +121,7 @@ def command_qemu(
 
     chroot = Chroot(ChrootType.ROOTFS, device)
     chroot_native = Chroot.native()
+    chroot_target = Chroot.buildroot(arch)
     flavor = pmb.chroot.other.kernel_flavor_installed(chroot, autoinstall=False)
     flavor_suffix = f"-{flavor}"
     # Backwards compatibility with old mkinitfs (pma#660)
@@ -131,9 +129,12 @@ def command_qemu(
     if pmaports_cfg.get("supported_mkinitfs_without_flavors", False):
         flavor_suffix = ""
 
+    # We do not support direct kernel boot on some architectures
+    use_direct_kernel_boot = arch.uses_direct_kernel_image_boot_by_default() and not args.efi
+
     # Alpine kernels always have the flavor appended to /boot/vmlinuz
     kernel = chroot / "boot" / f"vmlinuz{flavor_suffix}"
-    if not kernel.exists():
+    if use_direct_kernel_boot and not kernel.exists():
         kernel = kernel.with_name(f"{kernel.name}-{flavor}")
         if not os.path.exists(kernel):
             raise RuntimeError("failed to find the proper vmlinuz path")
@@ -207,14 +208,14 @@ def command_qemu(
                 ]
             )
         ]
-        command += [chroot_native / "usr/bin" / f"qemu-system-{arch.qemu()}"]
+        command += [chroot_native / "usr/bin" / f"qemu-system-{arch.qemu_system()}"]
         command += ["-L", chroot_native / "usr/share/qemu/"]
 
     command += ["-nodefaults"]
-    # Only boot a kernel/initramfs directly when not doing EFI boot. This
-    # allows us to load/execute an EFI application on boot, and support
+    # Only boot a kernel/initramfs directly when not doing EFI or OF boot.
+    # This allows us to load/execute an EFI application on boot, and support
     # a wide variety of boot loaders.
-    if not args.efi:
+    if use_direct_kernel_boot:
         command += ["-kernel", kernel]
         command += ["-initrd", chroot / "boot" / f"initramfs{flavor_suffix}"]
         command += ["-append", shlex.quote(cmdline)]
@@ -242,7 +243,7 @@ def command_qemu(
     command += ["-device", "virtio-net-pci,netdev=net"]
 
     # Check that arch is supported by pmb qemu
-    if arch not in [Arch.x86_64, Arch.aarch64, Arch.riscv64, Arch.ppc64le]:
+    if arch not in [Arch.x86_64, Arch.aarch64, Arch.riscv64, Arch.ppc64le, Arch.loongarch64]:
         raise RuntimeError(f"Architecture {arch} not supported by this command yet.")
 
     # Apply arch-specific qemu config
@@ -254,6 +255,8 @@ def command_qemu(
             command += ["-M", "virt"]
         case Arch.ppc64le:
             command += ["-M", "pseries"]
+        case Arch.loongarch64:
+            command += ["-M", "virt"]
 
     # Configure display
     if args.qemu_display != "none":
@@ -262,29 +265,41 @@ def command_qemu(
                 command += ["-device", "virtio-vga-gl"]
             case Arch.aarch64:
                 command += ["-device", "virtio-gpu-gl"]
-            case Arch.riscv64:
-                command += ["-device", "virtio-gpu-pci"]
-            case Arch.ppc64le:
+            case Arch.riscv64 | Arch.ppc64le | Arch.loongarch64:
                 command += ["-device", "virtio-gpu-pci"]
             case _:
                 # default to 2D-only backend
                 command += ["-device", "virtio-gpu"]
 
-    if args.efi:
-        host_arch = platform.machine().lower()
-        match host_arch:
-            case "aarch64":
+    if not use_direct_kernel_boot:
+        host_arch = Arch.native()
+        edk2_chroot = chroot_native.path if arch == host_arch else chroot_target.path
+        match arch:
+            case Arch.aarch64:
                 command += [
                     "-drive",
-                    f"if=pflash,format=raw,readonly=on,file={chroot_native.path}/usr/share/AAVMF/AAVMF_CODE.fd",
+                    f"if=pflash,format=raw,readonly=on,file={edk2_chroot}/usr/share/AAVMF/AAVMF_CODE.fd",
                 ]
-            case "amd64":
+            case Arch.x86_64:
                 command += [
                     "-drive",
-                    f"if=pflash,format=raw,readonly=on,file={chroot_native.path}/usr/share/OVMF/OVMF.fd",
+                    f"if=pflash,format=raw,readonly=on,file={edk2_chroot}/usr/share/OVMF/OVMF.fd",
+                ]
+            case Arch.loongarch64:
+                command += [
+                    "-drive",
+                    f"if=pflash,format=raw,readonly=on,file={edk2_chroot}/usr/share/edk2/loongarch64/QEMU_EFI.fd",
+                ]
+            case Arch.ppc64le:
+                # ppc64le uses SLOF included in QEMU
+                pass
+            case Arch.riscv64:
+                command += [
+                    "-drive",
+                    f"if=pflash,format=raw,readonly=on,file={edk2_chroot}/usr/share/edk2/riscv/RISCV_VIRT_CODE.fd",
                 ]
             case _:
-                raise RuntimeError(f"Architecture {host_arch} not configured for EFI support.")
+                raise RuntimeError(f"Architecture {arch} not configured for EFI support.")
 
     # Kernel Virtual Machine (KVM) support
     native = pmb.parse.deviceinfo().arch.is_native()
@@ -352,7 +367,7 @@ def resize_image(img_size_new: str, img_path: Path) -> None:
         raise RuntimeError(f"IMAGE_SIZE must be {img_size_str} or greater")
 
 
-def sigterm_handler(number: int, stack_frame: FrameType | None) -> None:
+def _sigterm_handler(number: int, stack_frame: FrameType | None) -> None:
     raise RuntimeError(
         "pmbootstrap was terminated by another process, and killed the QEMU VM it was running."
     )
@@ -376,7 +391,7 @@ def install_depends(args: PmbArgs, arch: Arch) -> None:
         "qemu-hw-display-virtio-gpu-pci-gl",
         "qemu-hw-display-virtio-vga",
         "qemu-hw-display-virtio-vga-gl",
-        "qemu-system-" + arch.qemu(),
+        "qemu-system-" + arch.qemu_system(),
         "qemu-ui-gtk",
         "qemu-ui-opengl",
         "qemu-ui-sdl",
@@ -390,15 +405,35 @@ def install_depends(args: PmbArgs, arch: Arch) -> None:
         depends.remove("qemu-hw-display-virtio-vga")
         depends.remove("qemu-ui-opengl")
 
-    if args.efi:
-        host_arch = platform.machine().lower()
-        match host_arch:
-            case "aarch64":
-                depends.append("aavmf")
-            case "amd64":
-                depends.append("ovmf")
+    use_direct_kernel_boot = arch.uses_direct_kernel_image_boot_by_default() and not args.efi
+
+    if not use_direct_kernel_boot:
+        edk2_pkg = None
+
+        # EDK2 builds are only available for the target architecture
+        match arch:
+            case Arch.aarch64:
+                edk2_pkg = "aavmf"
+            case Arch.x86_64:
+                edk2_pkg = "ovmf"
+            case Arch.loongarch64:
+                edk2_pkg = "edk2-loongarch64"
+            case Arch.ppc64le:
+                # ppc64le uses SLOF included in QEMU instead of EFI
+                pass
+            case Arch.riscv64:
+                edk2_pkg = "ovmf"
             case _:
-                raise RuntimeError(f"Architecture {host_arch} not configured for EFI support.")
+                raise RuntimeError(f"Architecture {arch} not configured for EFI support.")
+
+        # If we're running natively, install it to the native chroot, otherwise we need to
+        # install it in the target chroot
+        if edk2_pkg is not None and arch == Arch.native():
+            depends.append(edk2_pkg)
+        elif edk2_pkg is not None:
+            target_chroot = Chroot.buildroot(arch)
+            pmb.chroot.init(target_chroot)
+            pmb.chroot.apk.install([edk2_pkg], target_chroot)
 
     chroot = Chroot.native()
     pmb.chroot.init(chroot)
@@ -430,7 +465,7 @@ def run(args: PmbArgs) -> None:
 
     if not args.host_qemu:
         install_depends(args, arch)
-    logging.info("Running postmarketOS in QEMU VM (" + arch.qemu() + ")")
+    logging.info("Running postmarketOS in QEMU VM (" + arch.qemu_system() + ")")
 
     qemu, env = command_qemu(args, config, arch, img_path, img_path_2nd)
 
@@ -469,8 +504,8 @@ def run(args: PmbArgs) -> None:
     # Run QEMU and kill it together with pmbootstrap
     process = None
     try:
-        signal.signal(signal.SIGTERM, sigterm_handler)
-        process = pmb.helpers.run.user(qemu, output="tui", env=env)
+        signal.signal(signal.SIGTERM, _sigterm_handler)
+        process = pmb.helpers.run.user(qemu, output=RunOutputTypeDefault.TUI, env=env)
     except KeyboardInterrupt:
         # In addition to not showing a trace when pressing ^C, let user know
         # they can override this behavior:
